//////////////////////////////////////////////////////////////////////
// Breakout - with Christmas Themes 
// John Lavigne (tornadokick16@aol.com)
//
// Code modifyed from:
//   "Demo1 ARM9 Code - Based on an example shipped with NDSLIB.
//    Chris Double (chris.double@double.co.nz)"
//////////////////////////////////////////////////////////////////////

#include <nds.h>
#include <nds/arm9/console.h>
#include <stdio.h>
#include "levels.cpp"

enum GAME_STATE {INTRODUCTION = 0, HOLDING, PLAY, LOSE, WIN, TIMER};
GAME_STATE current_state = HOLDING, last_state = HOLDING;  // normally = intro

  int BG_Red = 0;   // sets the background color to black
  int BG_Green = 0;
  int BG_Blue = 0;
  static uint16 BGColor = RGB15(BG_Red,BG_Green,BG_Blue);

  static int shape_width = 32;
  static int shape_height = 4;
  static int brick_width = 16;
  static int brick_height = 4;

  static int paddle_old_x = 112;
  static int paddle_old_y = 180;
  static int paddle_shape_x = 112;
  static int paddle_shape_y = 180;

  static int ball_x = 126;
  static int delta_x = 0;
  static int ball_old_x = 126;
  static int ball_y = 176;
  static int delta_y = 0;
  static int ball_old_y = 176;

  static int level = 1;

  static int timer = 15;

  static int score = 0;
  static int combo_counter = 0;
  static int extra_lives = 0;
  static int lives = 3;
  
  static uint16 snowball[4][4];

void InitInterruptHandler();
void on_irq();
void draw_brick(int x, int y, uint16* buffer, uint16 color);
void draw_ball(int x, int y, uint16* buffer);
void draw_paddle(int x, int y, uint16* buffer, uint16 color);
void draw_background(uint16* buffer, uint16 color);
void draw_level(uint16* buffer, uint16 current_level[16][22]);
void calculateBall();
void getInput(uint16* buffer, uint16 current_level[16][22]);
void moveBall(uint16 current_level[16][22], int level, uint16* buffer);
void delete_ball(int x, int y, uint16* buffer);
void indicies(int *x, int *y);
void checkForHit(uint16* buffer, uint16 current_level[16][22]);
bool checkForWin(uint16 current_level[16][22]);
uint16 getColor(uint16 current_level[16][22]);
int combo_bonus();
uint16 getColor(int code);
void markHit(int x, int y, uint16 *buffer, uint16 *currentBlock);
void displayInfo(int level);
void killPaddle(uint16* buffer, uint16 color);
void createPaddle(uint16* buffer, uint16 color);

int main(void)
{
  powerON(POWER_ALL);

  videoSetMode(MODE_FB0);
  vramSetBankA(VRAM_A_LCD);

  videoSetModeSub(MODE_0_2D | DISPLAY_BG0_ACTIVE);
  vramSetBankC(VRAM_C_SUB_BG);
  SUB_BG0_CR = BG_MAP_BASE(31);
  BG_PALETTE_SUB[255] = RGB15(31,31,31);
  consoleInitDefault((u16*)SCREEN_BASE_BLOCK_SUB(31), (u16*)CHAR_BASE_BLOCK_SUB(0), 16);

  displayInfo(level);

    // lets the ARM9 know that interrupts will be used
    // and which ones
  InitInterruptHandler();
  
  draw_background(VRAM_A, BGColor);

  uint16 current_level[16][22];

  loadLevels(current_level, level);
  draw_level(VRAM_A, current_level);

  while(level <= MAX_LEVEL) {
      // this updates the current position of the ball
    ball_old_x = ball_x;
    ball_old_y = ball_y;

      // this updates the current position of the paddle
    paddle_old_x = paddle_shape_x;
    paddle_old_y = paddle_shape_y;

    last_state = current_state;

    getInput(VRAM_A, current_level);

      // this structure makes sure the paddle doesn't leave the screen
      // it will loop around instead
    if(paddle_shape_x + shape_width >= SCREEN_WIDTH) {
      paddle_shape_x = SCREEN_WIDTH - shape_width;
    }
    else if (paddle_shape_x < 0){
      paddle_shape_x = 0;
    }

    checkForHit(VRAM_A, current_level);
    moveBall(current_level, level, VRAM_A);

    swiWaitForVBlank();
  }

  while(1){;}
}

void InitInterruptHandler()
{
  REG_IME = 0;
  IRQ_HANDLER = on_irq;
  REG_IE = IRQ_VBLANK;
  REG_IF = ~0;
  DISP_SR = DISP_VBLANK_IRQ;
  REG_IME = 1;
}

  // stands for "On, interrupt request"
void on_irq() 
{	
  if(REG_IF & IRQ_VBLANK) {

      // this draws a paddle, but colors it black, thus deleting it
    draw_paddle(paddle_old_x, paddle_old_y, VRAM_A, BGColor);
      // this draws a new paddle
    if (lives > 0)
      draw_paddle(paddle_shape_x, paddle_shape_y, VRAM_A, RGB15(31, 0, 0));

      // this deletes, the old ball
    delete_ball(ball_old_x, ball_old_y, VRAM_A);

      // this draws a new ball
    if (lives > 0)
    draw_ball(ball_x, ball_y, VRAM_A);

    // Tell the DS we handled the VBLANK interrupt
    VBLANK_INTR_WAIT_FLAGS |= IRQ_VBLANK;
    REG_IF |= IRQ_VBLANK;
  }
  else {
    // Ignore all other interrupts
    REG_IF = REG_IF;
  }
}

void draw_ball(int x, int y, uint16* buffer)
{
  uint16* line;
  buffer += y * SCREEN_WIDTH + x;

  for (int i = 0; i < 4; i++){
    line = buffer + (SCREEN_WIDTH * i);
    for (int j = 0; j < 4; j++){
      *line++ = snowball[j][i];
    }
  }
}

void delete_ball(int x, int y, uint16* buffer)
{
  uint16* line;
  buffer += y * SCREEN_WIDTH + x;
  for(int i = 0; i < 4; ++i) {
    line = buffer + (SCREEN_WIDTH * i);
    for(int j = 0; j < 4; ++j, line++) {
      *line = BGColor;
    }
  }
}

// draws a single brick on the screen
void draw_brick(int x, int y, uint16* buffer, uint16 color)
{
  uint16* line;

  int midred = ( (color & RGB15(15,0,0)) + (BGColor & RGB15(15,0,0)) ) / 2;
  int midgreen = ( (color & RGB15(0,15,0)) + (BGColor & RGB15(0,15,0)) ) / 2;
  int midblue = ( (color & RGB15(0,0,15)) + (BGColor & RGB15(0,0,15)) ) / 2;
  uint16 middleColor = midred | midgreen | midblue;

  buffer += y * SCREEN_WIDTH + x;
  for(int i = 0; i < brick_height; ++i) {
    line = buffer + (SCREEN_WIDTH * i);
    for(int j = 0; j < brick_width-1; ++j, line++) {
      if (i == shape_height-1 || j == shape_width-1)
        *line = middleColor;
      else
        *line = color;
    }
  }
}

void draw_paddle(int x, int y, uint16* buffer, uint16 color)
{
  uint16* line;
  buffer += y * SCREEN_WIDTH + x;
  for(int i = 0; i < shape_height; ++i) {
    line = buffer + (SCREEN_WIDTH * i);
    for(int j = 0; j < shape_width; ++j, line++) {
      *line = color;
      if ((i == 0 || i == shape_height-1) 
        && (j == 0 || j == shape_width-1))
        *line = BGColor;
    }
  }
}

void draw_background(uint16* buffer, uint16 color)
{
  uint16* line;
  line = buffer;

  for (int i = 0; i < SCREEN_WIDTH * SCREEN_HEIGHT; i++)
    *(line++) = color;

  BGColor = color;

  calculateBall();
}

void draw_level(uint16* buffer, uint16 current_level[16][22])
{
  uint16 tempColor = RGB15(31,31,31);
  for (int i = 0; i < 22; i++){
    for (int j = 0; j < 16; j++){
      tempColor = getColor((int)current_level[j][i]);
      if (current_level[j][i] != 0)
        draw_brick(j * brick_width, 23 + i * brick_height, buffer, tempColor );
    }
  }
}

// adjusts the ball's sprite to the background
void calculateBall()
{
  int tempRed = 0, tempGreen = 0, tempBlue = 0;

  tempRed = (BG_Red*3 + 31) / 4;
  tempGreen = (BG_Green*3 + 31) / 4;
  tempBlue = (BG_Blue*3 + 31) / 4;
  snowball[0][0] = RGB15(tempRed, tempGreen, tempBlue);

  tempRed = (BG_Red + 29 + 31*14) / 16;
  tempGreen = (BG_Green + 29 + 31*14) / 16;
  tempBlue = (BG_Blue + 29 + 31*14) / 16;
  snowball[1][0] = RGB15(tempRed, tempGreen, tempBlue);
  snowball[0][1] = RGB15(tempRed, tempGreen, tempBlue);

  tempRed = (BG_Red + 26*3 + 29*6 + 31*6) / 16;
  tempGreen = (BG_Green + 26*3 + 29*6 + 31*6) / 16;
  tempBlue = (BG_Blue + 26*3 + 29*6 + 31*6) / 16;
  snowball[2][0] = RGB15(tempRed, tempGreen, tempBlue);
  snowball[0][2] = RGB15(tempRed, tempGreen, tempBlue);

  tempRed = (BG_Red*3 + 29) / 4;
  tempGreen = (BG_Green*3 + 29) / 4;
  tempBlue = (BG_Blue*3 + 29) / 4;
  snowball[3][0] = RGB15(tempRed, tempGreen, tempBlue);
  snowball[0][3] = RGB15(tempRed, tempGreen, tempBlue);
  snowball[3][3] = RGB15(tempRed, tempGreen, tempBlue);

  snowball[1][1] = RGB15(31,31,31);  // center of the snowball
  snowball[2][1] = RGB15(30,30,30);
  snowball[1][2] = RGB15(30,30,30);
  snowball[2][2] = RGB15(29,29,29);

  tempRed = (BG_Red + 26*3 + 29*12) / 16;
  tempGreen = (BG_Green + 26*3 + 29*12) / 16;
  tempBlue = (BG_Blue + 26*3 + 29*12) / 16;
  snowball[3][1] = RGB15(tempRed, tempGreen, tempBlue);
  snowball[1][3] = RGB15(tempRed, tempGreen, tempBlue);

  tempRed = (BG_Red + 26 + 29*14) / 16;
  tempGreen = (BG_Green + 26 + 29*14) / 16;
  tempBlue = (BG_Blue + 26 + 29*14) / 16;
  snowball[3][2] = RGB15(tempRed, tempGreen, tempBlue);
  snowball[2][3] = RGB15(tempRed, tempGreen, tempBlue);
}

void getInput(uint16* buffer, uint16 current_level[16][22])
{
  touchPosition touchXY = touchReadXY();

  uint16 keysPressed = ~(REG_KEYINPUT);

  if (keysPressed & KEY_START) {
    paddle_shape_x = 112;

    ball_x = 126;
    delta_x = 0;
    ball_y = 176;
    delta_y = 0;

    lives = 3;

    score = 0;
    combo_counter = 0;
    displayInfo(level);
    current_state = HOLDING;

    draw_background(buffer, BGColor);

    loadLevels(current_level, level);
    draw_level(buffer, current_level);
  }

  switch(current_state)
  {
  case INTRODUCTION:
    break;
  case HOLDING:
    if (touchXY.px != 0)
    {
      paddle_shape_x = touchXY.px - shape_width/2;
      ball_x = touchXY.px - 2;
    }
    if (keysPressed & KEY_A || keysPressed & KEY_L || keysPressed & KEY_R){
      timer = 25;
      current_state = TIMER;
    }
    break;
  case PLAY:
    if (touchXY.px != 0)
      paddle_shape_x = touchXY.px - shape_width/2;
    break;
  case LOSE:
    break;
  case WIN:
    break;
  case TIMER:
      --timer;
      if (timer == 0){
        current_state = PLAY;
        delta_x = 1;
        delta_y = 3;
      }
    break;
  default:
    break;
  };
}

void moveBall(uint16 current_level[16][22], int level, uint16* buffer)
{
    // move the ball horizontally
  ball_x += delta_x;

    // conditions on moving the ball horizontally
  if (ball_x >= SCREEN_WIDTH - 4){  
    ball_x = (SCREEN_WIDTH - 4) - (ball_x - (SCREEN_WIDTH - 4));
    delta_x = -delta_x;
  }
  else if (ball_x < 0){
    ball_x = -ball_x;
    delta_x = -delta_x;
  }

    // move the ball vertically
  ball_y -= delta_y;

    // conditions on moving the ball vertically, including off the paddle
  if (current_state == PLAY)
  {
    if (ball_y >= SCREEN_HEIGHT - 4){
      ball_y = (SCREEN_HEIGHT - 4) - (ball_y - (SCREEN_HEIGHT - 4));
      delta_y = -delta_y;
    }
    else if (ball_y >= paddle_shape_y-4 && ball_y < paddle_shape_y-1){  // remember, the length is 32
      if (ball_x >= paddle_shape_x - 2 && ball_x < paddle_shape_x + 2){  // low left
        delta_x = -3;
        delta_y = 1;
      }
      else if (ball_x >= paddle_shape_x + 2 && ball_x < paddle_shape_x + 6){  // middle left
        delta_x = -2;
        delta_y = 2;
      }
      else if (ball_x >= paddle_shape_x + 6 && ball_x < paddle_shape_x + 10){   // high left
        delta_x = -1;
        delta_y = 3;
       }
      else if (ball_x >= paddle_shape_x + 10 && ball_x < paddle_shape_x + 18){   // middle area
        if (delta_y < 0)
          delta_y = -delta_y;
      }
      else if (ball_x >= paddle_shape_x + 18 && ball_x < paddle_shape_x + 22){  // high right
        delta_x = 1;
        delta_y = 3;
      }
      else if (ball_x >= paddle_shape_x + 22 && ball_x < paddle_shape_x + 26){  // middle right
        delta_x = 2;
        delta_y = 2;
      }
      else if (ball_x >= paddle_shape_x + 26 && ball_x < paddle_shape_x + 30){   // low right
        delta_x = 3;
        delta_y = 1;
      }
      if (ball_x >= paddle_shape_x - 2 && ball_x < paddle_shape_x + 30 && ball_y > paddle_shape_y-4){
        ball_y += ball_y - paddle_shape_y;
      }

      score += combo_bonus();

      displayInfo(level);

      if (checkForWin(current_level)){
        level++;

        loadLevels(current_level, level);
        if (level <= MAX_LEVEL)
          draw_level(VRAM_A, current_level);
 
        draw_paddle(paddle_old_x, paddle_old_y, VRAM_A, BGColor);
        delete_ball(ball_old_x, ball_old_y, VRAM_A);
    
        paddle_old_x = 112;
        paddle_old_y = 180;
        paddle_shape_x = 112;
        paddle_shape_y = 180;

        ball_x = 126;
        delta_x = 1;
        ball_y = 176;
        delta_y = 3;
        
        lives++;

        current_state = HOLDING;
        displayInfo(level);
      }
    }
    else if (ball_y < 0){
      ball_y = -ball_y;
      delta_y = -delta_y;
    }
  }

    // checks for loss of life
  if (ball_y > paddle_shape_y + 2)
  {
    lives--;
    killPaddle(buffer, RGB15(31,0,0));
      
    paddle_shape_x = 112;
    paddle_shape_y = 180;

    if (lives > 0)
      createPaddle(buffer, RGB15(31,0,0));

    ball_x = 126;
    delta_x = 0;
    ball_y = 176;
    delta_y = 0;

    if (lives > 0) {
      current_state = HOLDING;
    }
    else
      current_state = LOSE;

    displayInfo(level);

    draw_background(buffer, BGColor);

    draw_level(buffer, current_level);
  }
}

void indicies(int *x, int *y)
{
   *y = (*y - 23)/4;
   *x = *x / 16;
}

void checkForHit(uint16* buffer, uint16 current_level[16][22])
{
  int x1 = ball_x + delta_x, y1 = ball_y - delta_y;
  int x2 = ball_x + delta_x + 3, y2 = ball_y - delta_y;
  int x3 = ball_x + delta_x, y3 = ball_y - delta_y + 3;
  int x4 = ball_x + delta_x + 3, y4 = ball_y - delta_y + 3;
  bool isValid1 = true, isValid2 = true, isValid3 = true, isValid4 = true;

  indicies(&x1, &y1);
  indicies(&x2, &y2);
  indicies(&x3, &y3);
  indicies(&x4, &y4);

  if ((x1 < 0 || x1 >= 16) || (y1 < 0 || y1 >= 22))
    isValid1 = false;
  if ((x2 < 0 || x2 >= 16) || (y2 < 0 || y2 >= 22))
    isValid2 = false;
  if ((x3 < 0 || x3 >= 16) || (y3 < 0 || y3 >= 22))
    isValid3 = false;
  if ((x4 < 0 || x4 >= 16) || (y4 < 0 || y4 >= 22))
    isValid4 = false;

  if (isValid1 == false &&
      isValid2 == false &&
      isValid3 == false &&
      isValid4 == false)
    return;

  if (isValid1 && current_level[x1][y1] > 0) {                  
    if (isValid2 && current_level[x2][y2] > 0) {                
      if (isValid3 && current_level[x3][y3] > 0) {     // note: can't be all four, yet
        if (delta_x < 0)
          delta_x = -delta_x;
        if (delta_y < 0)
          delta_y = -delta_y;
        markHit(x1,y1, buffer, &current_level[x1][y1]);
        markHit(x2,y2, buffer, &current_level[x2][y2]);
        markHit(x3,y3, buffer, &current_level[x3][y3]);
      }
      else {                                                    
        if (isValid4 && current_level[x4][y4] > 0) {            
          if (delta_x > 0)
            delta_x = -delta_x;
          if (delta_y < 0)
            delta_y = -delta_y;
          markHit(x1,y1, buffer, &current_level[x1][y1]);
          markHit(x2,y2, buffer, &current_level[x2][y2]);
          markHit(x4,y4, buffer, &current_level[x4][y4]);
        }
        else {                                                  
          delta_y = -delta_y;
          if (x1 == x2 && y1 == y2)
            markHit(x1,y1, buffer, &current_level[x1][y1]);            
          else {
            markHit(x1,y1, buffer, &current_level[x1][y1]);            
            markHit(x2,y2, buffer, &current_level[x2][y2]);            
          }          
        }
      }
    }
    else {   
      if (isValid3 && current_level[x3][y3] > 0) {             
        if (isValid4 && current_level[x4][y4] > 0) {            
          if (delta_x < 0)
            delta_x = -delta_x;
          if (delta_y > 0)
            delta_y = -delta_y;
          markHit(x1,y1, buffer, &current_level[x1][y1]);
          markHit(x3,y3, buffer, &current_level[x3][y3]);
          markHit(x4,y4, buffer, &current_level[x4][y4]);
        }
        else {
          delta_x = -delta_x;
          if (x1 == x3 && y1 == y3)
            markHit(x1,y1, buffer, &current_level[x1][y1]);            
          else{
            markHit(x1,y1, buffer, &current_level[x1][y1]);            
            markHit(x3,y3, buffer, &current_level[x3][y3]);
          }            
        }
      }
      else {                                                    
        if (isValid4 && current_level[x4][y4] > 0) {            
          delta_x = -delta_x;
          delta_y = -delta_y;
          markHit(x1,y1, buffer, &current_level[x1][y1]);            
          markHit(x4,y4, buffer, &current_level[x4][y4]);            
        }
        else {                                                  
          if (delta_x < 0)
            delta_x = -delta_x;
          if (delta_y > 0)
            delta_y = -delta_y;
            markHit(x1,y1, buffer, &current_level[x1][y1]);            
        }
      }
    }
  }
  else {                                                        
    if (isValid2 && current_level[x2][y2] > 0) {                
      if (isValid3 && current_level[x3][y3] > 0) {              
        if (isValid4 && current_level[x4][y4] > 0) {            
          if (delta_x > 0)
            delta_x = -delta_x;
          if (delta_y > 0)
            delta_y = -delta_y;
          markHit(x2,y2, buffer, &current_level[x2][y2]);
          markHit(x3,y3, buffer, &current_level[x3][y3]);
          markHit(x4,y4, buffer, &current_level[x4][y4]);
        }
        else {                                                  
          delta_x = -delta_x;
          delta_y = -delta_y;
          markHit(x2,y2, buffer, &current_level[x2][y2]);
          markHit(x3,y3, buffer, &current_level[x3][y3]);
        }
      }
      else {                                                    
        if (isValid4 && current_level[x4][y4] > 0) {            
          delta_x = -delta_x;
          if (x2 == x3 && y2 == y3)
            markHit(x2,y2, buffer, &current_level[x2][y2]);
          else {
            markHit(x2,y2, buffer, &current_level[x2][y2]);
            markHit(x3,y3, buffer, &current_level[x3][y3]);
          }
        }
        else {                                                  
          if (delta_x > 0)
            delta_x = -delta_x;
          if (delta_y > 0)
            delta_y = -delta_y;
          markHit(x2,y2, buffer, &current_level[x2][y2]);
        }
      }
    }
    else {                                                      
      if (isValid3 && current_level[x3][y3] > 0) {             
        if (isValid4 && current_level[x4][y4] > 0) {            
          delta_y = -delta_y;
          if (x3 == x4 && y3 == y4)
            markHit(x3,y3, buffer, &current_level[x3][y3]);
          else {
            markHit(x3,y3, buffer, &current_level[x3][y3]);
            markHit(x4,y4, buffer, &current_level[x4][y4]);
          }
        }
        else {                                                  
          if (delta_x < 0)
            delta_x = -delta_x;
          if (delta_y < 0)
            delta_y = -delta_y;
          markHit(x3,y3, buffer, &current_level[x3][y3]);
        }
      }
      else {                                                    
        if (isValid4 && current_level[x4][y4] > 0) {            
          if (delta_x > 0)
            delta_x = -delta_x;
          if (delta_y < 0)
            delta_y = -delta_y;
          markHit(x4,y4, buffer, &current_level[x4][y4]);
        }
      }
    }
  }
}

bool checkForWin(uint16 current_level[16][22])
{
  int i, j;
  for (i = 0; i < 22; i++)
    for (j = 0; j < 16; j++)
      if (current_level[j][i] != 0 && current_level[j][i] != 10)
        return false;
  return true;
}

int combo_bonus()
{
  int temp = 0;

  combo_counter -= 4;

  if (combo_counter <= 0){
    combo_counter = 0;
    return 0;
  }

  for (int i = 0; i < combo_counter; i++)
    temp += i;
  combo_counter = 0;

  return temp * 10;
}

uint16 getColor(int code)
{
  uint16 tempColor;

  switch(code)  // to choose the color
  {
  case 0:
    tempColor = BGColor;   // no block
    break;
  case 1:
    tempColor = RGB15(31,31,31);   // white
    break;
  case 2:
    tempColor = RGB15(31,0,0);     // red
    break;
  case 3:
    tempColor = RGB15(0,20,0);     // christmas green
    break;
  case 4:
    tempColor = RGB15(0,0,16);    // dark blue
    break;
  case 5:
    tempColor = RGB15(31,31,0);   // yellow
    break;
  case 6:
    tempColor = RGB15(16,8,0);    // brown
    break;
  case 7:
    tempColor = RGB15(14,0,29);   // purple
    break;
  case 8:
    tempColor = RGB15(31,15,15);   // pink
    break;
  case 9:
    tempColor = RGB15(10,30,9);   // orange
    break;
  case 10:
    tempColor = RGB15(10,10,10);   // grey
    break;
  default:
    tempColor = BGColor;
  };  
  return tempColor;
}

void markHit(int x, int y, uint16 *buffer, uint16 *currentBlock)
{
  uint16 tempColor;

  switch((int)*currentBlock)  // to choose the color
  {
  case 1:
  case 2:
  case 4:
    *currentBlock = 0;
    tempColor = BGColor;          // is not hit
    break;
  case 3:
    *currentBlock = 1;
    tempColor = RGB15(31,31,31);  // white
    break;
  case 5:
  case 7:
    *currentBlock = 2;
    tempColor = RGB15(31,0,0);    // red
    break;
  case 6:
    *currentBlock = 3;
    tempColor = RGB15(0,20,0);    // christmas green
    break;
  case 8:
    *currentBlock = 7;
    tempColor = RGB15(14,0,29);   // purple
    break;
  case 9:
    *currentBlock = 4;
    tempColor = RGB15(0,0,16);   // dark blue
    break;
  case 10:
    // color remains the same
    tempColor = RGB15(10,10,10);   // grey
    break;
  default:
    *currentBlock = 0;
    tempColor = BGColor;
  };  
  
  draw_brick(x * brick_width, 23 + y * brick_height, buffer, tempColor);
  score += 10;
  combo_counter++;

  if (score / 7500 > extra_lives)
  {
    lives += score/7500 - extra_lives;
    extra_lives = score / 7500;
  }

  displayInfo(level);
}

void displayInfo(int level)
{
  printf("\x1b[0m0;0H");
  printf("                               \n");
  printf("                               \n");
  printf("                               \n");
  printf("                               \n");
  printf("                               \n");
  printf("                               \n");
  printf("                               \n");
  printf("                               \n");
  printf("                               \n");
  printf("                               \n");
  printf("                               \n");
  printf("                               \n");
  printf("                               \n");
  printf("                               \n");
  printf("                               \n");
  printf("                               \n");
  printf("                               \n");
  printf("                               \n");


  printf("\x1b[0m0;0H");

  printf("\n\tHappy Holi-DS.\n\t\tby John Lavigne\n\n");

  printf("\tLevel #%d\n\t\t", level);
  switch(level)
  {
  case 1:
    printf("X-mas Tree       ");
    break;
  case 2:
    printf("Presents         ");
    break;
  case 3:
    printf("Chimney          ");
    break;
  case 4:
    printf("Advent Wreath    ");
    break;
  case 5:
    printf("Winter House     ");
    break;
  default:
    printf("N/A");
  };

  printf("\n\n\tScore:\n\t\t%d     \n\n", score);

  if (lives > 0)
    printf("\tCombo:      \n\t\t%d     \n\n", combo_counter);
  else
    printf("\tGAME OVER      \n          \n          \n");

  printf("\tLives:\n\t\t%d     \n\n", lives);

  printf("Press START for new game,\n");
  printf("then press A, L, or R to begin");
}

void killPaddle(uint16* buffer, uint16 color)
{
  int tempWidth;
  uint16* line;
  buffer += paddle_shape_y * SCREEN_WIDTH + paddle_shape_x;

  for (tempWidth = 32; tempWidth >= 0; tempWidth-=2) {
      // delete the current paddle
    draw_paddle(paddle_old_x, paddle_old_y, VRAM_A, BGColor);  

    for(int i = 0; i < shape_height; ++i) {
      line = buffer + (SCREEN_WIDTH * i);
      for(int j = 0; j < 16 - tempWidth/2; j++)
        line++;
      for(int j = 16 - tempWidth/2; j < tempWidth; ++j, line++) {
        *line = color;
        if ((i == 16 - tempWidth/2 || i == shape_height-1) 
          && (j == 16 - tempWidth/2 || j == tempWidth-1))
          *line = BGColor;
      }
    }
    swiWaitForVBlank();
  }
}

void createPaddle(uint16* buffer, uint16 color)
{
  int tempWidth;
  uint16* line;
  buffer += paddle_shape_y * SCREEN_WIDTH + paddle_shape_x;

  for (tempWidth = 0; tempWidth < 32; tempWidth+=2) {
    draw_paddle(paddle_shape_x, paddle_shape_y, VRAM_A, BGColor);  
    for(int i = 0; i < shape_height; ++i) {
      line = buffer + (SCREEN_WIDTH * i);
      for(int j = 0; j < 16 - tempWidth/2; j++)
        line++;
      for(int j = 16 - tempWidth/2; j < tempWidth; ++j, line++) {
        *line = color;
        if ((i == 16 - tempWidth/2 || i == shape_height-1) 
          && (j == 16 - tempWidth/2 || j == tempWidth-1))
          *line = BGColor;
      }
    }
    swiWaitForVBlank();
  }
}
